@@ -23,18 +23,20 @@ from tornado import gen, ioloop
 from tornado.concurrent import Future, future_set_exc_info, is_future
 
 import cognite.geospatial.internal
+from cognite.geospatial._primitive_spatial_object import PrimitiveGeometrySpatialObject
+from cognite.geospatial._raster_spatial_object import RasterGeometrySpatialObject
+from cognite.geospatial._spatial_filter_object import SpatialFilterObject
 from cognite.geospatial._spatial_object import SpatialObject
 from cognite.geospatial.internal import (
+    CoreGeometrySpatialItemDTO,
     CoreSpatialItemDTO,
     CreateSpatialItemsDTO,
     ExternalIdDTO,
     GeometryProjectionDTO,
     InternalIdDTO,
-    RasterGeometrySpatialDTO,
     SpatialDataDTO,
     SpatialDataRequestDTO,
     SpatialDataTypeDTO,
-    SpatialFilterDTO,
     SpatialIdsDTO,
     SpatialItemsProjectionDTO,
     SpatialSearchRequestDTO,
@@ -93,17 +95,25 @@ class CogniteGeospatialClient:
     """
 
     def __init__(
-        self, api_key: str = None, base_url: str = None, port: int = None, project: str = None, timeout: int = 60000
+        self,
+        api_key: str = None,
+        base_url: str = None,
+        port: int = None,
+        api_token: str = None,
+        project: str = None,
+        timeout: int = 60000,
     ):
         # configure env
         api_key = api_key or os.getenv("COGNITE_API_KEY")
-        if api_key is None or not api_key.strip():
+        if (api_key is None or not api_key.strip()) and api_token is None:
             raise ValueError(
                 "You have either not passed an api key or not set the COGNITE_API_KEY environment variable."
             )
         self.configuration = cognite.geospatial.internal.Configuration()
         self.configuration.client_side_validation = False
-        self.configuration.api_key["api-key"] = api_key.strip()
+        if api_token is None:
+            self.configuration.api_key["api-key"] = api_key.strip()
+        self.configuration.access_token = api_token
 
         base_url = base_url or "api.cognitedata.com"
         port = port or 443
@@ -333,7 +343,7 @@ class CogniteGeospatialClient:
         if geography is not None:
             geometry = self._create_geometry(geography=geography)
             proj = _create_projection(projection)
-            spatial_filter = SpatialFilterDTO(
+            spatial_filter = SpatialFilterObject(
                 "intersection", geometry, projection=proj, local_vars_configuration=self.configuration
             )
         spatial_search_request = SpatialSearchRequestDTO(
@@ -404,17 +414,18 @@ class CogniteGeospatialClient:
         geometry = self._create_geometry(id, external_id, geography)
         proj = _create_projection(projection)
         if distance is None:
-            spatial_filter = SpatialFilterDTO(
+            spatial_filter = SpatialFilterObject(
                 "within", geometry=geometry, projection=proj, local_vars_configuration=self.configuration
             )
         else:
-            spatial_filter = SpatialFilterDTO(
+            spatial_filter = SpatialFilterObject(
                 "within_distance",
                 geometry=geometry,
                 projection=proj,
                 distance_meter=distance,
                 local_vars_configuration=self.configuration,
             )
+
         spatial_search_request = SpatialSearchRequestDTO(limit=limit, spatial_filter=spatial_filter, crs=crs)
 
         try:
@@ -461,7 +472,7 @@ class CogniteGeospatialClient:
         project = self._get_project(project)
         geometry = self._create_geometry(id, external_id, geography)
         proj = _create_projection(projection)
-        spatial_filter = SpatialFilterDTO(
+        spatial_filter = SpatialFilterObject(
             "completely_within", geometry, projection=proj, local_vars_configuration=self.configuration
         )
         spatial_search_request = SpatialSearchRequestDTO(limit=limit, spatial_filter=spatial_filter, crs=crs)
@@ -491,6 +502,56 @@ class CogniteGeospatialClient:
         result = self._run_sync(run_func, self.timeout)
         return result
 
+    @gen.coroutine
+    def create_spatial_async(
+        self,
+        name,
+        type: str,
+        geometry: str,
+        crs: str,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+    ):
+        geometry = PrimitiveGeometrySpatialObject(type=type, wkt=geometry, local_vars_configuration=self.configuration)
+        spatial_item = CoreGeometrySpatialItemDTO(
+            name=name,
+            external_id=external_id,
+            description=description,
+            metadata=metadata,
+            source=source,
+            geometry=geometry,
+            crs=crs,
+        )
+
+        create_spatial_items = CreateSpatialItemsDTO(items=[spatial_item])
+        try:
+            response = yield self.api.create_spatial(self.project, create_spatial_items)
+            if response is not None:
+                return response.items
+            return None
+        except ApiException as e:
+            _throw_exception(e)
+
+    def create_spatial(
+        self,
+        name,
+        type: str,
+        geometry: str,
+        crs: str,
+        external_id: str = None,
+        description: str = None,
+        metadata=None,
+        source: str = None,
+    ):
+
+        run_func = partial(
+            self.create_spatial_async, name, type, geometry, crs, external_id, description, metadata, source
+        )
+        result = self._run_sync(run_func, self.timeout)
+        return result
+
     @gen.coroutine
     def create_raster_spatial_async(
         self,
@@ -505,14 +566,14 @@ class CogniteGeospatialClient:
         column_min: int = None,
         column_max: int = None,
     ):
-        geometry = RasterGeometrySpatialDTO(
+        geometry = RasterGeometrySpatialObject(
             row_min=row_min,
             row_max=row_max,
             column_min=column_min,
             column_max=column_max,
             local_vars_configuration=self.configuration,
         )
-        spatial_item = CoreSpatialItemDTO(
+        spatial_item = CoreGeometrySpatialItemDTO(
             name=name,
             external_id=external_id,
             description=description,
